@@ -83,8 +83,7 @@ typedef struct DHPart {
     uint8_t hashH1[8*ZRTP_WORD_SIZE];
     uint8_t rs1Id[2*ZRTP_WORD_SIZE];
     uint8_t rs2Id[2*ZRTP_WORD_SIZE];
-//    uint8_t sigsId[2*ZRTP_WORD_SIZE];
-    uint8_t s3Id[2*ZRTP_WORD_SIZE];
+    uint8_t auxSecretId[2*ZRTP_WORD_SIZE];
     uint8_t pbxSecretId[2*ZRTP_WORD_SIZE];
 }  DHPart_t;
 
